package Mentat::Script;
use strict;
use warnings;

#*******************************************************************************
# DOCUMENTATION SECTION
#*******************************************************************************

=head1 NAME

Mentat::Script

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 COPYRIGHT

This program is free software; you can redistribute
it and/or modify it under the same terms as Perl itself.

The full text of the license can be found in the
LICENSE file included with this module.


=head1 SEE ALSO

perl(1).

=head1 FUNCTION REFERENCE

=over 4

=cut

#*******************************************************************************
# LIBRARY LOADING SECTION
#*******************************************************************************

#-- Perl core modules ---------------------------------------------------------#
use Carp;
use Time::HiRes qw(time);

use Data::Dumper; #-+-> DEVEL ONLY <-+-#
#use Smart::Comments; #-+-> DEVEL ONLY <-+-#

#-- Perl CPAN modules ---------------------------------------------------------#
use JSON;

#-- Custom application modules ------------------------------------------------#
use App::Env;
use App::Env::Scanner;
use Log::Loger;
use Value::Convertor;
use Mentat::Core qw(:func :c_general);
use Mentat::App;

eval "use Mentat::Storage::Mongo;";

#*******************************************************************************
# CONSTANTS AND GLOBAL VARIABLES DEFINITION SECTION
#*******************************************************************************

#-- Constants -----------------------------------------------------------------#

#-- Static public class variables (our) ---------------------------------------#

#-- Static protected class variables (my) -------------------------------------#

my $RUNLOG;

#*******************************************************************************
# INITIALIZATION AND CLEANUP SECTION
#*******************************************************************************

#-- Module initializations ----------------------------------------------------#
BEGIN {
    use vars qw($VERSION $DEVEL);
    $VERSION = '0.1';
    $DEVEL   = 0;
}

#-- Module clean-up code (global destructor) ----------------------------------#
END {

}

#*******************************************************************************
# IMPLEMENTATION SECTION
#*******************************************************************************

sub init(@)
{
    my ($env_config,) = @_;

    Mentat::App::init($env_config);

    $RUNLOG = {
            'app_name' => app_name(),
            'ts_start' => time,
        };

    return $RUNLOG;
}

sub meta_load { return json_load(@_); }
sub meta_save { return json_save(@_); }

sub msg($)
{
    my ($msg,) = @_;

    print STDERR "$msg" unless App::Env->plugin;
}

sub dbg($$)
{
    my ($label, $obj) = @_;

    print STDERR "$label: ".Dumper($obj);
}

sub store_log(%)
{
    my %cfg = @_;

    eval {
        my $storage = Mentat::Storage::Mongo->new(database => $cfg{database}, collection => $cfg{collection});
        my $id = $storage->insert($RUNLOG);
        msg("Logged runlog into database '$cfg{database}:$cfg{collection}' in document '$id'\n");
    };
    if ($@) {
        msg("Unable to store runlog into database '$cfg{database}:$cfg{collection}': '$@'\n");
    }
}

sub dump_log(%)
{
    my %cfg = @_;

    my $json = to_json($RUNLOG, {utf8 => 1, pretty => 1});

    open(my $hnd, ">", $cfg{dump_file}) or die "Can`t open dump file '$cfg{dump_file}' for writing\n";
    print $hnd $json."\n";
    close($hnd);
    msg("Dumped runlog into in file '$cfg{dump_file}'\n");
}

sub finish(%)
{
    my %cfg = @_;

    $RUNLOG->{'ts'}       = time;
    $RUNLOG->{'duration'} = $RUNLOG->{ts} - $RUNLOG->{ts_start};

    if (App::Env->store_log) {
        store_log(%cfg);
    }
    if (App::Env->dump_log) {
        dump_log(%cfg);
    }
    if (App::Env->plugin) {
        print to_json($RUNLOG, {utf8 => 1, pretty => 1})."\n";
    }
}

=pod

=back

=cut

1;
